c Copyright (c) 2013, Dr Renato Vacondio, Dr Benedict Rogers, Prof. Peter Stansby 
c and Prof. Paolo Mignosa
c 
c All rights reserved.
c 
c This file is part of SWE-SPHYSICS.
c 
c Redistribution and use in source and binary forms, with or without modification, 
c are permitted provided that the following conditions are met:
c 
c - Redistributions of source code must retain the above copyright notice, 
c  this list of conditions and the following disclaimer.
c - Redistributions in binary form must reproduce the above copyright notice, 
c  this list of conditions and the following disclaimer in the documentation 
c  and/or other materials provided with the distribution.
c
c THE "SWE-SPHYSICS" IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
c ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
c OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
c THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
c SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
c OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
c HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
c TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
c EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
c
c    Newton - Raphson iterative procedure for water depth and smoothing length calculations
c   

      subroutine ac_dw
c
      use global_1D

      integer i_hmaxdef, ncou_max, i, j1 
      real Res, hmax_def, adh, dxx, ncou, pippo

      i_hmaxdef=0
      ncou_max=0
      hmax_def=0

      do i=nstart,np_fixed !1

          if (mod(itime,i_dw_iter).eq.0) then
             Res_adm(i)=tol+1
          else
             Res_adm(i)=0
          endif

         if (iflag(i).gt.0) then !2

c --- initial guess               
             rhop(i)=rhop(i)+dt*rhop(i)*ar(i)
             h_var(i)=h_var(i)-(dt/dm)*h_var(i)*ar(i)

c      -- check hmax and re-assign variables if (h(i) > hmax) --
            if (h_var(i)>hmax) then
               n_assign=n_assign+1
               call grid_h_var(i)
            end if

c        -- calculating position of particle i --
               dxx = xp(i) - xmin
               j1 = int( dxx*one_over_2hmax ) + 1
            
         ncou=0

c --- iterative loop --- 
         do while ( (Res_adm(i) .gt. tol).and.(ncou.le.i_max_iter) ) !3

             ncou=ncou+1
             if (ncou.gt.ncou_max) ncou_max=ncou

c        -- Zeroing Variables --
             rhop_sum=0 !we dont store it, it is useless
             alphap(i)=0       
    
c        -- calculating rhop_sum and alpha(i)
             if (j1.lt.ncx) then
                   call celij_dw(i,j1,j1+1,2)
                   call celij_dw_ob(i,j1,j1+1)
             endif

             if (j1.gt.1) then
                   call celij_dw(i,j1,j1-1,2)
                   call celij_dw_ob(i,j1,j1-1)
             endif

             call self_dw(i,j1,2)
             call celij_dw_ob(i,j1,j1)
                                      
            !-- self contribution -- 

                adh=( 2/( 3*h_var(i) ) )   ! cubic kernel
                rhop_sum=rhop_sum+pm(i)*adh

c     -- END: calculating rhop_sum and alphap(i)

            Res=rhop(i)-rhop_sum !residual
            Res_adm(i) = abs(Res/rhop(i))       !adimensional residual

            !density k+1     
            rhop(i)=rhop(i)*( 1 -( Res*dm/( Res*dm + alphap(i))))

            if (rhop(i).le.0) then
                rhop(i)=rhop_sum
                write(*,*) 'WARNING problem with the iterative 
     + procedure at particle:', i, 'positioned at:', xp(i), 
     +'rhop=',rhop(i), 'rhop_sum=', rhop_sum, 'h_var(i)=',h_var(i)
            endif

            !smoothing length k+1
            h_var(i) = h_var0(i)*( rho0*dw0(i)/rhop(i) )**(1/dm)

c      -- check hmax and re-assign variables if (h(i) > hmax) --
            if (h_var(i)>hmax) then
c              write(*,*) 'i=',i,' hmax =', h_var(i)
               n_assign=n_assign+1
               call grid_h_var(i)
               
               !new cell for particle i
               dxx = xp(i) - xmin
               j1 = int( dxx*one_over_2hmax ) + 1
            end if

         end do !3 
c --- END of iterative loop ---
         endif !2

c     -- update water depth --
         dw(i)=rhop(i)/rho0

         if (h_var(i).gt.hmax_def) then        !find the particle with hmax
            i_hmaxdef=i
            hmax_def=h_var(i)
         endif

      enddo !1
      write(*,*) 'max iteration number is', ncou_max
      if (ncou_max.gt.i_max_iter) then
          write(*,*) 'WARNING: maximum number of iterations reached, 
     + increase the parameter i_max_iter and re-run the simulation'
      endif


c      -- check hmax and re-assign variables --
         if (i_hmaxdef.gt.0) then
             n_assign=n_assign+1
             call grid_h_var(i_hmaxdef)
         endif
        
c      -- alphap with update h_var for every particles --
       call ac_alpha

      end
