c Copyright (c) 2013, Dr Renato Vacondio, Dr Benedict Rogers, Prof. Peter Stansby 
c and Prof. Paolo Mignosa
c 
c All rights reserved.
c 
c This file is part of SWE-SPHYSICS.
c 
c Redistribution and use in source and binary forms, with or without modification, 
c are permitted provided that the following conditions are met:
c 
c - Redistributions of source code must retain the above copyright notice, 
c  this list of conditions and the following disclaimer.
c - Redistributions in binary form must reproduce the above copyright notice, 
c  this list of conditions and the following disclaimer in the documentation 
c  and/or other materials provided with the distribution.
c
c THE "SWE-SPHYSICS" IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
c ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
c OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
c THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
c SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
c OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
c HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
c TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
c EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	subroutine check_limits_1D
	use global_1D

        integer i, iflagg, ncases

	ncases=0

	do i=nbp1,np ! Check fluid particles
 
	   if ( iflag(i).ge.1.and.(xp(i).gt.xmax.or.
     +      xp(i).lt.xmin) ) then
     
              iflag(i)=0
	      ncases=ncases+1
	      xp(i)=xtrash
	      up(i)=0.
	      uo(i)=0.
	      
	   elseif (iflag(i).eq.1 .and. xp(i).gt.xmax_container) then !fluid particle is exiting downstream
	   
	      if   (iflagbc(2).eq.4) then
	          iflagg=4 !inflow
	      elseif (iflagbc(2).eq.5) then
	          iflagg=5 !outflow
	      endif
	      
	      call new_ob_flp( i,2,iflagg ) !add one bc particle downstream with same coordinate of the fluid particle
	      
	      xp(i)=xtrash
	      up(i)=0.
	      uo(i)=0.
	      iflag(i)=0
	         
	   elseif (iflag(i).eq.1 .and. xp(i).lt.xmin_container) then !fluid particle is exiting upstream
	   
	      if   (iflagbc(1).eq.2) then
	          iflagg=2 !inflow
	      elseif (iflagbc(1).eq.3) then
	          iflagg=3 !outflow
	      endif
	      
	      call new_ob_flp( i,1,iflagg ) !add one bc particle upstream with same coordinate of the fluid particle
	       	      
	      xp(i)=xtrash
	      up(i)=0.
	      uo(i)=0.
	      iflag(i)=0

           endif   
	enddo
	
	do i=1,np_ob ! Check bc particles
	   
	   if ( iflag_ob(i).ge.1.and.(xp_ob(i).gt.xmax_ob.or.
     +      xp_ob(i).lt.xmin_ob) ) then
     
              iflag_ob(i)=0              
	      xp_ob(i)=xtrash	      
	      up_ob(i)=0.	      
	      uo_ob(i)=0.
	      
	   elseif (
     +   (iflag_ob(i).gt.1 .and. iflag_ob(i).le.3 .and.
     + xp_ob(i).ge.xmin_container .and. 
     + xp_ob(i).le.xmax_container) ) then !bc is entering upstream
           
              call new_fl_ob(i) !it generates a new fluid particle from the bc particle
              call new_bcp(1,xp_ob(i)) !it generates a new bc particle
              iflag_ob(i)=0
              xp_ob(i)=xtrash	      
	      up_ob(i)=0.	      
	      uo_ob(i)=0.
              
           elseif ( 
     +    (iflag_ob(i).gt.3 .and.  iflag_ob(i).le.5 .and. 
     +     xp_ob(i).ge.xmin_container .and.
     +     xp_ob(i).le.xmax_container) ) then !bc is entering downstream
       
              call new_fl_ob(i) !it generates a new fluid particle from the bc particle 
              call new_bcp(2,xp_ob(i)) !it generates a new bc particle
              iflag_ob(i)=0
              xp_ob(i)=xtrash	      
	      up_ob(i)=0.	      
	      uo_ob(i)=0.
              
           endif
	
	enddo	
	
      if (ncases.ne.0) write(*,*) 'Number ',ncases,' time',time

      return
      
      end
      
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ 

c-----------------------------------------------------------------------
c it generates a new bc particle when a bc particle becomes a fluid particle
c ipos=1 then upstream, ipos=2 then downstream
c xpos position of the new fluid particle just entered into the domain
c-----------------------------------------------------------------------
            
       subroutine new_bcp(ipos,xpos)
       use global_1D

       integer ipos, i
       real xpos, dummy
      
       i=np_ob+1
      
       iflag_ob(i)=iflagbc(ipos)
       ib(i)=i
       xp_ob(i)=xpos-2*dxobc(ipos)*verbc(ipos)
       h_var_ob(i)=h_varbc(ipos)
       call open_bc(i,i,dummy) !< it calculates up, uo and dw using sph interpolation
c       dwo_ob(i)=dw_ob(i) 
       dw0_ob(i)=dw0bc(ipos) 
       pm_ob(i)=dw_ob(i)*dxobc(ipos)*rho0  !dwbc(ipos)*dxobc(ipos)*rho0
       h_var0_ob(i)=h_varbc(ipos)/2.
       rhop_ob(i)=dw_ob(i)*rho0
       cs_ob(i)=sqrt(grav*dw_ob(i))
       alphap_ob(i)=rhop_ob(i)*dm
c       alphapo_ob(i)=alphap_ob(i)
       
       np_ob=i
       
       return

       end 
      
      
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^           
c-----------------------------------------------------------------------
c it generates a new bc particle from a fluid particle
c ifl is the index of the fluid particle
c ipos to define is it is upstream or downstream
c iflag to define which kind of bc we are considering
c-----------------------------------------------------------------------


      subroutine new_ob_flp(ifl,ipos,iflagg)
      use global_1D

      integer iflagg, ifl, ipos, i
      
       i=np_ob+1
      
       xp_ob(i)=xp(ifl)
       pm_ob(i)=pm(ifl)
       h_var_ob(i)=2*h_var(ifl)       
       dw_ob(i)=dw(ifl)
       up_ob(i)=up(ifl)
       uo_ob(i)=uo(ifl)
c       dwo_ob(i)=dwo(ifl)     
       rhop_ob(i)=dw_ob(i)*rho0
       cs_ob(i)=sqrt(grav*dw_ob(i))
       alphap_ob(i)=alphap(ifl)
c       alphapo_ob(i)=alphap_ob(i)
       ib(i)=i
       iflag_ob(i)=iflagg
       dw0_ob(i)=dw0(ifl)
       h_var0_ob(i)=h_var0(ifl)

       np_ob=i
       
       return
       
       end
       
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^  
      
c-----------------------------------------------------------------------
c it generates a new fluid particle from a bc particle
c iob is the index of the bc particle
c-----------------------------------------------------------------------      
       subroutine new_fl_ob(iob)
       use global_1D

       integer iob, icell, ii
       real dxx 
       
       np=np+1
       np_fixed=np
       
       xp(np)=xp_ob(iob)
       pm(np)=pm_ob(iob)
       h_var(np)=h_var_ob(iob)/2.
       h_var0(np)=h_var0_ob(iob)
       up(np)=up_ob(iob) 
       uo(np)=uo_ob(iob) 
       dw(np)=dw_ob(iob)
       dw0(np)=dw0_ob(iob)
c       dwo(np)=dw_ob(iob)    
       rhop(np)=dw(np)*rho0
       cs(np)=sqrt(grav*dw(np))
       alphap(np)=rhop(np)*dm
c       alphapo(np)=alphap(np)
       iflag(np)=1
       
       !it assigns the fluid particle to his cell
       dxx = xp(np) - xmin   
       icell = int( dxx * one_over_2hmax ) + 1
       ii    = icell
       nc(ii,2) = nc(ii,2)+1	
                                      
       if(nc(ii,2).gt.nplink_max)then
           print*
           print*,'ERROR in new_fl_ob'
           print*,'nc(ii,kind_p) >= nplink_max'
           print*,'itime',itime
           print*,'nc(ii,kind_p)',nc(ii,2)
           print*,'kind_p       ',2
           print*,'nplink_max',nplink_max
           print*,'k',np
           print*,'xp(k)',xp(np)
           print*,'icell',icell
           print*,'Box ii ',ii
           stop
       end if
          
      ibox( ii,2,nc(ii,2) )=np
    
       return
       
       end
      
      
