c Copyright (c) 2013, Dr Renato Vacondio, Dr Benedict Rogers, Prof. Peter Stansby 
c and Prof. Paolo Mignosa
c 
c All rights reserved.
c 
c This file is part of SWE-SPHYSICS.
c 
c Redistribution and use in source and binary forms, with or without modification, 
c are permitted provided that the following conditions are met:
c 
c - Redistributions of source code must retain the above copyright notice, 
c  this list of conditions and the following disclaimer.
c - Redistributions in binary form must reproduce the above copyright notice, 
c  this list of conditions and the following disclaimer in the documentation 
c  and/or other materials provided with the distribution.
c
c THE "SWE-SPHYSICS" IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
c ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
c OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
c THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
c SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
c OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
c HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
c TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
c EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


c
c SPH interpolation with corr kernel over fluid particles to obtain u_i and dw_i
c for particles in open boundary area
c      
      
      subroutine open_bc(ini,ifin,sm_maxob)

      use global_1D

      real sm_maxob, dxx, sqdw
      integer ini, ifin, i, j1

      sm_maxob=0.
      
      do i=ini,ifin !1
         if (iflag_ob(i).gt.1) then !2  !just for boundary condition particles
         
         ! zeroing variables
           upi(i)=0
           uoi(i)=0
           dwi(i)=0
           sum_wab(i)=0
           
c        -- calculating position of particle i --
           dxx = xp_ob(i) - xmin
           j1 = int( dxx*one_over_2hmax ) + 1

            if(j1+1.le.ncx)then !6
               call celij_openbc(i,j1,j1+1)     
            endif      !6
                  
            if(j1+2.le.ncx)then !6
               call celij_openbc(i,j1,j1+2)    
            endif      !6
                  
            if(j1-1.ge.1)then !6
               call celij_openbc(i,j1,j1-1)    
            endif      !6
                  
            if(j1-2.ge.1)then !6
               call celij_openbc(i,j1,j1-2)    
            endif      !6
           
           call celij_openbc(i,j1,j1)

           if (sum_wab(i).gt.0.) then         
               upi(i)=upi(i)/sum_wab(i)
               uoi(i)=uoi(i)/sum_wab(i)
               dwi(i)=dwi(i)/sum_wab(i)
            else
c               write(*,*) 'WARNING: open bc i=',i,'xp_ob(i)=',xp_ob(i), 
c     + 'has no interaction with fluid particles'       
               dwi(i)=dw0_ob(i)
           endif
           
           !imposition of bc upstream
           if (iflag_ob(i).eq.2) then !inflow upstream !3
           
               up_ob(i)=upbc(1)
               uo_ob(i)=upbc(1)
               
               if ( up_ob(i).gt.sqrt(grav*dwi(i)) ) then
                   dw_ob(i)=dwbc(1) !supercritical flow
c                   write(*,*) 'sup inflow upstream for particle i', i, 
c     + 'xp_ob(i)=', xp_ob(i), 'dw_ob=', dw_ob(i), 'up_ob=', up_ob(i)
               else
                   sqdw=(0.5/sqrt(grav))*(upi(i)-upbc(1))+sqrt(dwi(i))
                   dw_ob(i)=sqdw*sqdw
c                   write(*,*) 'sub inflow upstream for particle i', i, 
c     + 'xp_ob(i)=', xp_ob(i), 'dw_ob=', dw_ob(i), 'up_ob=', up_ob(i)
               endif
           
           elseif (iflag_ob(i).eq.3) then !outflow upstream
           
               if ( upi(i).gt.sqrt(grav*dwi(i)) ) then
                   dw_ob(i)=dwi(i)
                   up_ob(i)=upi(i) !supercritical flow
                   uo_ob(i)=uoi(i) 
c                   write(*,*) 'sup outflow upstream for particle i', i, 
c     + 'xp_ob(i)=', xp_ob(i), 'dw_ob=', dw_ob(i), 'up_ob=', up_ob(i)
               else
                   dw_ob(i)=dwbc(1)
                   up_ob(i)=upi(i)-2*sqrt(grav)*verbc(1)*
     +              ( sqrt(dwi(i))-sqrt(dwbc(1)) )
                   uo_ob(i)=uoi(i)-2*sqrt(grav)*verbc(1)*
     +              ( sqrt(dwi(i))-sqrt(dwbc(1)) )
c                   write(*,*) 'sub outflow upstream for particle i', i, 
c     + 'xp_ob(i)=', xp_ob(i), 'dw_ob=', dw_ob(i), 'up_ob=', up_ob(i)
               endif
               
           elseif (iflag_ob(i).eq.4) then !inflow downstream     
               
               up_ob(i)=upbc(2)
               uo_ob(i)=upbc(2)
               
               if (up_ob(i).gt.sqrt(grav*dwi(i)) ) then
                   dw_ob(i)=dwbc(2) !supercritical flow
c                   write(*,*) 'sup inflow downstream for particle i', i, 
c     + 'xp_ob(i)=', xp_ob(i), 'dw_ob=', dw_ob(i), 'up_ob=', up_ob(i)
               else
                   sqdw=(0.5/sqrt(grav))*(upi(i)-upbc(2))+sqrt(dwi(i))
                   dw_ob(i)=sqdw*sqdw
c                   write(*,*) 'sub inflow downstream for particle i', i, 
c     + 'xp_ob(i)=', xp_ob(i), 'dw_ob=', dw_ob(i), 'up_ob=', up_ob(i)
               endif
               
           elseif (iflag_ob(i).eq.5) then !outflow downstream
           
               if ( upi(i).gt.sqrt(grav*dwi(i)) ) then
c                   write(*,*) 'particle i=', i, 'xp_ob(i)=',xp_ob(i),
c     + 'supercritical outflow'
                   dw_ob(i)=dwi(i)
                   up_ob(i)=upi(i) !supercritical flow
                   uo_ob(i)=uoi(i)
c                  write(*,*) 'sup outflow downstream for particle i', i, 
c     + 'xp_ob(i)=', xp_ob(i), 'dw_ob=', dw_ob(i), 'up_ob=', up_ob(i)
               else
c                   write(*,*) 'particle i=', i, 'xp_ob(i)=',xp_ob(i),
c     + 'subcritical outflow'
                   dw_ob(i)=dwbc(2)
                   up_ob(i)=upi(i)-2*sqrt(grav)*verbc(2)*
     +              ( sqrt(dwi(i))-sqrt(dwbc(2)) )
                   uo_ob(i)=uoi(i)-2*sqrt(grav)*verbc(2)*
     +              ( sqrt(dwi(i))-sqrt(dwbc(2)) )
c                  write(*,*) 'sub outflow downstream for particle i', i, 
c     + 'xp_ob(i)=', xp_ob(i), 'dw_ob=', dw_ob(i), 'up_ob=', up_ob(i)
               endif
               
           endif !3
           
           cs_ob(i)=sqrt(dw_ob(i)*grav)
           rhop_ob(i)=rho0*dw_ob(i)
           alphap_ob(i)=rhop_ob(i)*dm
           sm_maxob=max( sm_maxob,h_var_ob(i) )

  
         endif !2
         
      enddo !1
      
      end subroutine
      
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^      

      subroutine celij_openbc(i,j1,j2)
      
      use global_1D

      integer i, j1, j2, jj, j
      real drx, rr2
      
          if(nc(j2,2).ne.0) then !1
         
          do jj=1,nc(j2,2) !2
             j = ibox(j2,2,jj)

            drx = xp_ob(i) - xp(j)
            rr2 = drx*drx

	    if( ( rr2.lt.( 4*h_var_ob(i)*h_var_ob(i) ) ) !3
     + .and.iflag(j).eq.1.and.rr2.gt.1.E-18) then !5 

c            Calculating kernel & Normalized Kernel

	       call kernel(const_ker,drx,h_var_ob(i),frx,fac,Wab)   

               upi(i)=upi(i)+up(j)*pm(j)/rhop(j)*Wab
               uoi(i)=uoi(i)+uo(j)*pm(j)/rhop(j)*Wab
               dwi(i)=dwi(i)+dw(j)*pm(j)/rhop(j)*Wab
               
               sum_wab(i)=sum_wab(i)+pm(j)/rhop(j)*Wab
               
            endif
            
          enddo
          endif
               
      end subroutine
      
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^      
