c Copyright (c) 2013, Dr Renato Vacondio, Dr Benedict Rogers, Prof. Peter Stansby 
c and Prof. Paolo Mignosa
c 
c All rights reserved.
c 
c This file is part of SWE-SPHYSICS.
c 
c Redistribution and use in source and binary forms, with or without modification, 
c are permitted provided that the following conditions are met:
c 
c - Redistributions of source code must retain the above copyright notice, 
c  this list of conditions and the following disclaimer.
c - Redistributions in binary form must reproduce the above copyright notice, 
c  this list of conditions and the following disclaimer in the documentation 
c  and/or other materials provided with the distribution.
c
c THE "SWE-SPHYSICS" IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
c ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
c OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
c THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
c SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
c OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
c HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
c TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
c EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

ccccccccccccccccccccccccccccccc
c	Leap-frog Method      c
ccccccccccccccccccccccccccccccc

      subroutine step
      use global_1D

      integer i
c
c  ---  time step ---
c
      
      if (ivar_dt.eq.1) then
	    call variable_time_step
	    dt=dt_new
	    !to plot result at precise timesteps
	    if (grab_P+dt.gt.out) then
	        dt=out-grab_p
	    endif
            if (itime.eq.0) then
               dt_old=dt
            endif
      endif

c
c  ---  accelerations ---
c
      call ac
      call source_slope
	
       	 
c
c  ---  velocities n+1/2,velocities v n+1  ---
c
      do i=1,np
         if (iflag(i).eq.1) then
             uo(i) = uo(i) + udot(i)*(dt + dt_old)*0.5 !vel n+1/2
             up(i) = uo(i) + 0.5*udot(i)*dt !vel n+1
         endif
      enddo
      
c --- vel and dw for open boundary particles ---      
      
      call open_bc(1,np_ob,hmax_ob)
      call open_bc_pos
      
c
c  ---  fluid particle positions n+1 ---
c
      
      do i=1,np
         xp(i) = xp(i) + uo(i)*dt  !position n+1
      enddo

      np=np_fixed
      call ini_divide(2) !< added for openbc condition
      call divide(nbp1,np,2)	
      call check_limits_1D	

c      
c --- open boundaries: it re-numbering the particles
c 
        if (mod(itime,100).eq.0) then      
           call recount	
	endif           
c
c  ---  link list and ghost particles ---
c
        call ghost
	call ini_divide(2)
	call divide(nbp1,np,2)	
	call ini_divide(3)
	call divide_ob(1,np_ob,3)
	
c
c  ---  bottom elevation for all particles ---
c	
      if (np_b.gt.0) then
         call bottom
      endif        
c
c  ---  update h_var, dw, rhop, alhphap, cs ---
c 

c       do i=1,np
c	alphapo(i)=alphap(i)
c	dwo(i)=dw(i)
c       enddo
       
       n_assign=0
       call ac_dw !computing rhop, dw, h_var, alphap 
       
       call open_bc(1,np_ob,hmax_ob)
       xmax_ob=xmax_container+hmax_ob
       xmin_ob=xmin_container-hmax_ob         

       do i=1,np
          cs(i)=sqrt(dw(i)*grav)  !computing cs !<=========  INSERT THIS IN AC_DW
       enddo

       dt_old=dt
       !np=np_fixed
        

      !just for debugging it writes results at every timestep
      if (idebug.eq.1) then
         open(23,file='tempris.txt')
         do i=1,np
            write(23,100) xp(i),up(i),dw(i)+h_t(i),rhop(i),ax(i),
     + t_st(i),alphap(i)/(rhop(i)*dm),Res_adm(i),h_t(i), pm(i)
         enddo
         close(23)
       
         open(23,file='tempris_ob.txt')
         do i=1,np_ob
            write(23,100) xp_ob(i),up_ob(i),dw_ob(i),rhop_ob(i),
     + h_var_ob(i),alphap_ob(i)/(rhop_ob(i)*dm)
         enddo
         close(23)
      endif

100      format(10e16.8)

      end

ccccccccccccccccccccccccccccccccc
c endo of Leap-frog Method      c
ccccccccccccccccccccccccccccccccc

