c Copyright (c) 2013, Dr Renato Vacondio, Dr Benedict Rogers, Prof. Peter Stansby 
c and Prof. Paolo Mignosa
c 
c All rights reserved.
c 
c This file is part of SWE-SPHYSICS.
c 
c Redistribution and use in source and binary forms, with or without modification, 
c are permitted provided that the following conditions are met:
c 
c - Redistributions of source code must retain the above copyright notice, 
c  this list of conditions and the following disclaimer.
c - Redistributions in binary form must reproduce the above copyright notice, 
c  this list of conditions and the following disclaimer in the documentation 
c  and/or other materials provided with the distribution.
c
c THE "SWE-SPHYSICS" IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
c ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
c OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
c THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
c SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
c OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
c HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
c TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
c EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

      program SPHYSICSgen_1D

      implicit none
      
      integer, parameter :: npar=7000
      
      character (LEN=40) :: file_bedProfile
     
      real xp(npar),pm(npar),dw(npar),rhop(npar),up(npar),cs(npar),
     + h_var(npar), h_t(npar),h
      real xb(npar), hb(npar), hb_SPH(npar)
      real xpbc(2), dwbc(2), upbc(2), pmbc(2), h_varbc(2), verbc(2),
     + dw0bc(2),dxobc(2)
     
      integer iflag(npar), iflagbc(2)

      integer i, i_bedType, i_compile_opt, i_dw_iter, i_max_iter, 
     + i_restartRun, idebug, idouble, if_openbc, iMUSCL, itest, ivar_dt, 
     + ivisc, np, np_b 

      real areap(npar), CFL, coef, dt, dxb, g, grav, hsm_b, out, 
     + pi, rho0, tmax, tol, trec_ini, viscos_val, vlx

       print*
       write(*,*) '<SWE-SPHYSICS>  Copyright (C) <2013>'
       write(*,*) 
     & '<Dr Renato Vacondio, Dr Benedict Rogers, '
       write(*,*) 
     & 'Prof. Peter Stansby and Prof. Paolo Mignosa, '
       write(*,*) 'This program comes with ABSOLUTELY NO WARRANTY;    '
       write(*,*) 'This is free software, and you are welcome to      '
       write(*,*) 'redistribute it under conditions stated in         '
       write(*,*) 'the FreeBSD License;                               '

       print*
       print*,' ---       SPHYSICS_SWE_1D.F       ---'
       print*,' ---       Distributed under       ---'
       print*,' ---      the FreeBSD License      ---'
       print*

! RV 17/05/10: we need to put the restart file the bc variable in time and the roughness. <========= TO DO !!!!!!

c       constants
c
      g=9.81
      grav=g
      pi=4.0*atan(1.0)

      write(*,*) 'Start a new RUN (0) or Restart an old RUN (1) ? '
      read(*,*) i_restartRun
      write(*,*) i_restartRun

      write(*,*) 'debug activated: 1 - for yes (output at every timestep)
     + 0 for no'
      read(*,*) idebug
      write(*,*) idebug
      
      write(*,*) 'fluid density (1000 kg/m3 for water):'
      read(*,*) rho0
      write(*,*) rho0

      write(*,*) 'stabilization term 1 - artificial viscosity, 
     + 2 - Lax Friedrichs flux, 3 - two shocks Riemann solver'
      read(*,*) ivisc
      write(*,*) ivisc

      write(*,*) 'valule of alpha for artificial viscosity'
      read(*,*) viscos_val  !2. 
      write(*,*) viscos_val  !2. 

      write(*,*) '0 - No MUSCL reconstruction, 1 - MUSCL reconstruction)
     +'
      read(*,*) iMUSCL
      write(*,*) iMUSCL

      write(*,*) 'tolerance in the Newton - Raphson algorithm
     + (suggested 1.E-3)'
      read(*,*) tol
      write(*,*) tol

      write(*,*) 'number of time step for Newton - Raphson iterative 
     + procedure (suggested 10) if 1 the iterative procedure is operated 
     + every time steps'
      read(*,*) i_dw_iter
      write(*,*) i_dw_iter

      write(*,*) 'maximum number of iterations in the Newton - Raphson  
     + iterative procedure'
      read(*,*) i_max_iter
      write(*,*) i_max_iter

      write(*,*) 'variable time step: 1 for yes, 2 for no'
      read(*,*) ivar_dt
      write(*,*) ivar_dt 

      write(*,*) ' time step, or maximum time step if variable time 
     + step activated (s)'
      read(*,*) dt !=0
      write(*,*) dt !=0  

      write(*,*) 'Courant number'
      read(*,*) CFL
      write(*,*) CFL

      write(*,*) 'output interval (seconds)'
      read(*,*) out
      write(*,*) out

      write(*,*) 'End of the simulation (seconds)'
      read(*,*) tmax  
      write(*,*) tmax   

      write(*,*) 'time step for intitial output (seconds)'
      read(*,*) trec_ini 
      write(*,*) trec_ini 

      write(*,*) 'smoothing length coefficient: h=dx*coef 
     + (suggested 1.2)'
      read(*,*) coef
      write(*,*) coef

      write(*,*) 'length of the domain (m)'
      read(*,*) vlx
      write(*,*) vlx

!open boundaries
      write(*,*) 'open boundaries: 0 for no, 2 for downstream, 
     + 1 for upstream, 3 for both'
      read(*,*) if_openbc
      write(*,*) if_openbc

      if (if_openbc.eq.1.or.if_openbc.eq.3) then
        write(*,*) 'kind of open boundary upstream: 2 for inflow 
     + or 3 - for outflow' 
        read(*,*) iflagbc(1)
        write(*,*) iflagbc(1)

        write(*,*) 'dx for upstream bc (should be equal to the one used 
     + for fluid particles) (m)' 
        read(*,*) dxobc(1)
        write(*,*) dxobc(1)

        xpbc(1)=-0.5*dxobc(1)

        write(*,*) 'water surface elevation for upstream bc (m)' !<=========TO DO: we should put here variable wse in time
        read(*,*)  dwbc(1)
        write(*,*) dwbc(1)

        write(*,*) 'velocity for upstream bc (m/s)' !<=========TO DO: we should put here variable wse in time
        read(*,*)  upbc(1)
        write(*,*) upbc(1)

        pmbc(1)=dxobc(1)*rho0*dwbc(1) !mass for obc particles
        h_varbc(1)=2*coef*dxobc(1) !variable smoothing length
        verbc(1)=1.
        dw0bc(1)=dwbc(1)

      endif

      if (if_openbc.eq.2.or.if_openbc.eq.3) then 
         write(*,*) 'kind of open boundary downstream: 4 for inflow 
     + or 5 - for outflow' 
         read(*,*) iflagbc(2)
         write(*,*) iflagbc(2)

        write(*,*) 'dx for downstream bc (should be equal to the one use
     + d for fluid particles) (m)' 
        read(*,*) dxobc(2)
        write(*,*) dxobc(2)

        xpbc(2)=vlx+0.5*dxobc(2)

        write(*,*) 'water surface elevation for downstream bc (m)' !<=========TO DO: we should put here variable wse in time
        read(*,*)  dwbc(2)
        write(*,*) dwbc(2)

        write(*,*) 'velocity for downstream bc (m/s)' !<=========TO DO: we should put here variable wse in time
        read(*,*)  upbc(2)
        write(*,*) upbc(2)

        pmbc(2)=dxobc(2)*rho0*dwbc(2) !mass for obc particles
        h_varbc(2)=2*coef*dxobc(2) !variable smoothing length
        verbc(2)=-1.
        dw0bc(2)=dwbc(2)

      endif

C     Define Bed Profile
      write(*,*) ' Choose Bed Profile  ??'
      write(*,*) ' 1=Plane Slope '
      write(*,*) ' 2=Parabolic, 3=Load Bed Profile from File'
      read(*,*) i_bedType
      write(*,*) i_bedType
      if(i_bedType.eq.1)then !plane slope
        call slopingBed(npar, np_b, dxb, hsm_b, xb, hb_SPH, coef, vlx)
      elseif(i_bedType.eq.2)then !parabolic slope
        call parabolicBed(npar, np_b, dxb, hsm_b, xb, hb_SPH, coef, vlx)
      elseif(i_bedType.eq.3)then
        print*
        write(*,*) ' Enter filename of Bed Profile'		
        read(*,*) file_bedProfile
        write(*,*) file_bedProfile
        call read_bedProfile(file_bedProfile, npar, np_b, dxb, hsm_b, 
     + xb, hb_SPH, coef, vlx)
      elseif(i_bedType.ne.0)then
        print*
        print*,'Incorrect Choice for Bed Profile'
        stop        
      endif

      !initial conditions
      call surface_define(npar, np, xp, dw, up,areap)
          
      if (np.gt.npar) then
          write(*,*) 'np=',np, '> npar, increase npar.'
          stop
      endif

      if (np_b.gt.npar) then
          write(*,*) 'np_b=',np_b,'> npar, increase npar.'
          stop
      endif
      
      open(75,file='indat')      
       write(75,*) itest
       write(75,*) rho0
       write(75,*) viscos_val
       write(75,*) coef
       write(75,*) vlx
       write(75,*) np
       write(75,*) np_b
       write(75,*) tol
       write(75,*) ivar_dt
       write(75,*) dt
       write(75,*) CFL
       write(75,*) tmax
       write(75,*) out
       write(75,*) trec_ini
       write(75,*) i_restartRun      
       write(75,*) dxb   
       write(75,*) hsm_b !smoothing length for the bottom particles
       write(75,*) if_openbc
       write(75,*) idebug
       write(75,*) iMUSCL
       write(75,*) i_dw_iter
       write(75,*) i_max_iter
       close(75)


        open(76,file='ipart')
        do i=1,np
           write(76,*) xp(i),up(i),dw(i),areap(i)
           write(*,*)  xp(i),up(i),dw(i),areap(i)
        enddo
        close (76) 
        
        open(77,file='ibottom')
        do i=1,np_b
           write(77,*) i,xb(i),hb_SPH(i)
        enddo
        close (77) 
       
        open(78,file='ibc')
        do i=1,2
           write(78,*) xpbc(i), pmbc(i), h_varbc(i), iflagbc(i), 
     +      dwbc(i), upbc(i), verbc(i), dw0bc(i),dxobc(i)
        enddo
        close (78)   

      write(*,*) 'Which compiler is desired? '
      write(*,*) '(1=gfortran, 2=ifort, 3=CVF, 4=Silverfrost FTN95)'
      read(*,*) i_compile_opt
      write(*,*) i_compile_opt

      write(*,*) '1 for single precision, 2 for double precision'
      read(*,*) idouble
      write(*,*) idouble

      if(i_compile_opt.eq.1) then
        call tocompile_unixStyle(i_compile_opt,ivisc,idebug,
     &                           idouble,iMUSCL)
      elseif(i_compile_opt.eq.2) then
        call tocompile_unixStyle(i_compile_opt,ivisc,idebug,
     &                           idouble,iMUSCL)
      elseif(i_compile_opt.eq.3) then
        call tocompile_cvf(ivisc,idebug,idouble,iMUSCL)
      else  !if(i_compile_opt.eq.4) then
        call tocompile_unixStyle(i_compile_opt,ivisc,idebug,
     &                           idouble,iMUSCL)
      end if
      
      end program   !--- end of initial data for SPHYSICS_SWE_1D ---
c
c***********************************************************************
c      
c     _______________SUBROUTINE definition of water surface elevation

      subroutine surface_define(npar, np, xp, dw, up, areap)

      implicit none

      integer npar, np, npoint, i, k, kk, iblock, np_block
      real xp(npar), dw(npar), up(npar), areap(npar)

      real xp_block(2), dw_block(2), up_block(2),  area_block, dx

      iblock=1

      np=0 
      kk=0

      do while (iblock.eq.1) 
         write(*,*) ' x-coordinate for the beginning of the block (m) '
         read(*,*)   xp_block(1)
         write(*,*)  xp_block(1)
         write(*,*) ' x-coordinate for the end of the block (m) '
         read(*,*)   xp_block(2)
         write(*,*)  xp_block(2)
         write(*,*) ' water surface elevation at the beginning of 
     + the block (m)'
         read(*,*)   dw_block(1)
         write(*,*)  dw_block(1)
         write(*,*) ' water surface elevation at the end of the block 
     + (m)'
         read(*,*)   dw_block(2)
         write(*,*)  dw_block(2)
         write(*,*) ' velocity at the beginning of the block (m/s)'
         read(*,*)   up_block(1)
         write(*,*)  up_block(1)
         write(*,*) ' velocity at the end of the block (m/s)' 
         read(*,*)   up_block(2)
         write(*,*)  up_block(2)
         write(*,*) ' particle spacing for this block (m)' 
         read(*,*)   area_block
         write(*,*)  area_block

         np_block=int( ( xp_block(2)-xp_block(1) )/area_block )+1 !number of particles in the portion of the domain
         dx= ( xp_block(2)-xp_block(1) )/np_block
         do i=1,np_block !loop over particles in block k
              kk=kk+1
              xp(kk)=xp_block(1)+(i-0.5)*dx
              call interp( xp(kk),dw(kk),xp_block,dw_block ) !free surface elevation 
              call interp( xp(kk),up(kk),xp_block,up_block ) !velocity 
               areap(kk)=dx
         enddo
         np=np+np_block

         write(*,*) ' Add another block (1=yes)'
         read(*,*) iblock
         write(*,*) iblock
      enddo


      end subroutine
c
c***********************************************************************
c      
c     ____________________ SUBROUTINE slopingBed

      subroutine slopingBed(npar, np_b, dxb, hsm_b, xb, hb_SPH, coef, 
     + vlx)

      implicit none

      integer np_b, npar, i
      real dxb, hsm_b, xb(npar), hb_SPH(npar), vlx
      real coef, b_elev0, xini,beta_deg, beta, pi

      pi=4.0*atan(1.0)

      write(*,*) 'distance between bottom particles (m):'
      read(*,*) dxb
      write(*,*) dxb

      hsm_b=coef*dxb

      np_b=int(vlx/dxb)+1
      dxb=vlx/np_b

      write(*,*) 'from 0 to xini the bottom is flat, xini (m):'
      read(*,*) xini
      write(*,*) xini

      if (xini.gt.vlx) then
          write(*,*) 'ERROR in slopingBed: xini.gt. vlx' 
          STOP
      endif

      write(*,*) 'beach elevation between 0 and xini (m):'
      read(*,*) b_elev0
      write(*,*) b_elev0

      write(*,*) 'beach slope (degree):'
      read(*,*) beta_deg
      write(*,*) beta_deg
      beta=beta_deg*pi/180

      do i=1,np_b

         xb(i)=(i-0.5)*dxb

         if (xb(i).le.xini) then
             hb_SPH(i)=b_elev0
         else
             hb_SPH(i)=b_elev0+(xb(i)-xini)*tan(beta)
         endif
         write(*,*) xb(i), hb_SPH(i)
      enddo

      end subroutine
c
c***********************************************************************
c

c     ____________________ SUBROUTINE parabolicBed

      subroutine parabolicBed(npar, np_b, dxb, hsm_b, xb, hb_SPH, coef, 
     + vlx)
      
      implicit none
      
      integer np_b, npar, i
      real dxb, hsm_b, xb(npar), hb_SPH(npar), coef, vlx
      real x0, y0, h0, a, hb_try

      write(*,*) 'distance between bottom particles (m):'
      read(*,*) dxb
      write(*,*) dxb

      hsm_b=coef*dxb

      np_b=int(vlx/dxb)+1
      dxb=vlx/np_b

      write(*,*) 'parabola equation y-y0=h0*[(x-x0)/a]**2, x0 parameter 
     + (m)'
      read(*,*) x0
      write(*,*) x0

      write(*,*) 'parabola equation y-y0=h0*[(x-x0)/a]**2, h0 parameter 
     + (m)'
      read(*,*) y0
      write(*,*) y0

      write(*,*) 'parabola equation y-y0=h0*[(x-x0)/a]**2, a parameter 
     + (m)'
      read(*,*) a
      write(*,*) a

      write(*,*) 'parabola equation y-y0=h0*[(x-x0)/a]**2, h0 parameter 
     + (m)'
      read(*,*) h0
      write(*,*) h0

      do i=1,np_b
         xb(i)=(i-0.5)*dxb
         hb_try=y0+h0*((xb(i)-x0)/a)**2 !parabola equation
         if (hb_try.gt.0) then
             hb_SPH(i)=hb_try
         else
             hb_SPH(i)=0
         endif
         write(*,*) xb(i), hb_SPH(i)
      enddo

      end subroutine
c
c***********************************************************************
c      
c     ____________________ SUBROUTINE read_bedProfile___________________
c
c it reads the bottom elevation from file file_bedProfile and calculate the elevation of bottom particles
c using a linear interpolation from data file. 
c xdata in the file have to be strictly increasing
c

      subroutine read_bedProfile(file_name, npar, np_b, dxb, 
     + hsm_b, xb, hb_SPH, coef, vlx)

      implicit none

      character (LEN=40) :: file_name

      integer npar, np_b, npoint, i, iallo, k
      real dxb, hsm_b, xb(npar), hb_SPH(npar), coef, vlx

      real, allocatable :: xpoint(:), ypoint(:)

      write(*,*) 'distance between bottom particles (m):'
      read(*,*) dxb
      write(*,*) dxb

      hsm_b=coef*dxb

      np_b=int(vlx/dxb)+1
      dxb=vlx/np_b

      !read data from file

      open(36,file=file_name, status='old')

      read(36,*) npoint
      write(*,*) 'npoint bed profile =', npoint

      allocate(xpoint(npoint), STAT=iallo)
      allocate(ypoint(npoint), STAT=iallo)

      do i=1,npoint
         read(36,*) xpoint(i), ypoint(i)
         write(*,*) 'xpoint,ypoint =', xpoint(i), ypoint(i)
      enddo
      close(36)

      !check that xpoint is increasing:
      if (xpoint(1).gt.0) then
          write(*,*) 'ERROR: first point in file', file_name , 
     +               'should have x-coordinate le 0'
          STOP
      endif

      do i=2,npoint
         if (xpoint(i).le.xpoint(i-1)) then
          write(*,*) 'ERROR: xdata in file', file_name , 
     +               'are not increasing'
          STOP
         endif
      enddo

      if (xpoint(npoint).lt.vlx) then
          write(*,*) 'ERROR: last point in file', file_name , 
     +               'should have coordinate ge than', vlx
          STOP
      endif

      !linearly interpolate hb_SPH using data from file
      write(*,*) 'bottom coordinates'
      write(*,*) 'xb(i), hb(i)'
      do i=1,np_b
         xb(i)=(i-0.5)*dxb
         k=1
         do while ( xb(i).lt.xpoint(k).or. xb(i).ge.xpoint(k+1) )
            k=k+1
         enddo
         call interp(xb(i),hb_SPH(i),xpoint(k:k+1),ypoint(k:k+1))
         write(*,*) xb(i), hb_SPH(i)
      enddo

      end subroutine
c
c***********************************************************************
c
c     ____________________ SUBROUTINE interp(x,y,xdata,ydata,n)

      subroutine interp(x,y,xdata,ydata)

      implicit none

      real x, y, xdata(2), ydata(2), ang_coef

      ang_coef=( ydata(2)-ydata(1) )/( xdata(2)-xdata(1) )
      
      y=ydata(1)+ang_coef*(x-xdata(1))

      end subroutine
c
c***********************************************************************


c     ____________________ SUBROUTINE TOCOMPILE_UNIXSTYLE


      subroutine tocompile_unixStyle(i_compile_opt,ivisc,idebug,
     &                               idouble,iMUSCL)

      CHARACTER(LEN=10) :: FMT,FMT1
      CHARACTER(LEN=1)  :: TAB
      CHARACTER(LEN=4)  :: OBJFINAL
      CHARACTER(LEN=6)  :: OBJ2
      CHARACTER(LEN=9)  :: obj_string
      TAB=CHAR(9)

      if(i_compile_opt.eq.1)then
        print*,'Using gfortran for unix/linux '
      elseif(i_compile_opt.eq.2)then
        print*,'Using Intel ifort for unix/linux '
      elseif(i_compile_opt.eq.4)then
        print*,'Using SilverFrost FTN95 '
      endif

	 
      FMT="(A)"
      FMT1="(2A)"
      

      
      open(22,file='SPHYSICS.mak')
      
      if(i_compile_opt.eq.1)then
        !- Using gfortran for unix/linux -
        OBJFINAL = '.o'
        OBJ2 = OBJFINAL//' \'
        obj_string = 'objects='
        write(22,FMT) 'FC=gfortran'
        if (idouble.eq.2) then  !Use Double Precision
          if (idebug.eq.1) then
            write(22,FMT) 'OPTIONS= -g -O0 '
     &        //' -fdefault-real-8'
     &        //' -ffpe-trap=invalid,zero,denormal'
      !- Debugging options, Uncomment as required -
c     &        //' -ftree-vectorize'
c     &        //' -ffast-math -funroll-loops'
c     &        //' -fbounds-check'
c     &        //' -frange-check'
c     &        //' -Wunderflow'
c     &        //' -Wuninitialized'
c     &        //' -ffpe-trap=invalid,zero,overflow'
c     &        //' -fstack-check'
c     &        //' -fstack-protector'
c     &        //' -ftrapv'
c     &        //' -ftracer'
c     &        //' -g'
c     &        //' -fimplicit-none'
c     &        //' -fno-automatic'
          else
            write(22,FMT) 'OPTIONS= -O3 -fdefault-real-8'
          endif
        else                    !Use Single Precision
           if (idebug.eq.1) then
            write(22,FMT) 'OPTIONS= -g -O0 '
     &        //' -ffpe-trap=invalid,zero,denormal'
       !- Debugging options, Uncomment as required -
c     &        //' -ftree-vectorize'
c     &        //' -ffast-math -funroll-loops'
c     &        //' -fbounds-check'
c     &        //' -frange-check'
c     &        //' -Wunderflow'
c     &        //' -Wuninitialized'
c     &        //' -ffpe-trap=invalid,zero,overflow'
c     &        //' -fstack-check'
c     &        //' -fstack-protector'
c     &        //' -ftrapv'
c     &        //' -ftracer'
c     &        //' -g'
c     &        //' -fimplicit-none'
c     &        //' -fno-automatic'
         else
            write(22,FMT) 'OPTIONS= -O3'
          endif
        endif

        write(22,FMT) 'srcdir=.'
        write(22,FMT) 'idir=../../execs'
        write(22,FMT) 'bakdir=../../execs.bak'

      elseif(i_compile_opt.eq.2)then

        !- Using Intel IFORT for unix/linux -
        OBJFINAL = '.o'
        OBJ2 = OBJFINAL//' \'
        obj_string = 'objects='
        write(22,FMT) 'FC=ifort'
        if (idouble.eq.2) then  !Use Double Precision
          if (idebug.eq.1) then
            write(22,FMT) 'OPTIONS= -g -O0 -r8 -fpe0'
      !- Debugging options, Uncomment as required -
     &       //' -ipo'
c     &       //' -g -debug all -check all'
c     &       //' -implicitnone'
c     &       //' -warn unused'
c     &       //' -fp-stack-check -heap-arrays'
c     &       //' -ftrapuv'
c     &       //' -check pointers'
c     &       //' -check bounds'
c     &       //' -check uninit'
c     &       //' -fpstkchk'
c     &       //' -traceback'
          else
            write(22,FMT) 'OPTIONS= -03 -r8'
     &       //' -ipo'
          endif
        else                    !Use Single Precision
          if (idebug.eq.1) then
            write(22,FMT) 'OPTIONS= -g -O0 -fpe0'
      !- Debugging options, Uncomment as required -
     &       //' -ipo'
c     &       //' -g -debug all -check all'
c     &       //' -implicitnone'
c     &       //' -warn unused'
c     &       //' -fp-stack-check -heap-arrays'
c     &       //' -ftrapuv'
c     &       //' -check pointers'
c     &       //' -check bounds'
c     &       //' -check uninit'
c     &       //' -fpstkchk'
c     &       //' -traceback'
          else
            write(22,FMT) 'OPTIONS= -fast'
          endif
        endif
        write(22,FMT) 'srcdir=.'
        write(22,FMT) 'idir=../../execs'
        write(22,FMT) 'bakdir=../../execs.bak'
      elseif(i_compile_opt.eq.4)then
        !- Using SilverFrost FTN95 -
        OBJFINAL = '.obj'
        OBJ2 = OBJFINAL//' \'
        obj_string = 'OBJFILES='
        if (idouble.eq.2) then  !Use Double Precision
          if (idebug.eq.1) then
            write(22,FMT) 'OPTIONS= /CHECK'
          else
            write(22,FMT) 'OPTIONS= /OPTIMISE'
          endif
        else                    !Use Single Precision
          if (idebug.eq.1) then
            write(22,FMT) 'OPTIONS= /CHECK'
          else
            write(22,FMT) 'OPTIONS= /OPTIMISE'
          endif
        endif
      endif
            
      write(22,FMT)
      write(22,FMT) obj_string//' global_1D'//OBJ2
      write(22,FMT1)TAB,'ini_divide_1D'//OBJ2
      write(22,FMT1)TAB,'SPHYSICS_SWE_1D'//OBJ2
      write(22,FMT1)TAB,'getdata_1D'//OBJ2
      write(22,FMT1)TAB,'check_limits_1D'//OBJ2
      write(22,FMT1)TAB,'poute_1D'//OBJ2
      write(22,FMT1)TAB,'divide_1D'//OBJ2
      write(22,FMT1)TAB,'variable_time_step_1D'//OBJ2
      write(22,FMT1)TAB,'ac_1D'//OBJ2
      write(22,FMT1)TAB,'ac_dw_var_1D'//OBJ2
      write(22,FMT1)TAB,'celij_dw_1D'//OBJ2
      write(22,FMT1)TAB,'self_dw_1D'//OBJ2
      write(22,FMT1)TAB,'kernel_cubic_1D'//OBJ2
      write(22,FMT1)TAB,'step_leap_frog_1D'//OBJ2
      write(22,FMT1)TAB,'ghost_1D'//OBJ2
      write(22,FMT1)TAB,'ac_alpha_1D'//OBJ2
      write(22,FMT1)TAB,'celij_alpha_1D'//OBJ2
      write(22,FMT1)TAB,'self_alpha_1D'//OBJ2
      write(22,FMT1)TAB,'grid_h_var_1D'//OBJ2
      
      if (ivisc.eq.3) then
         write(22,FMT1)TAB,'celij_balsara_1D'//OBJ2
         write(22,FMT1)TAB,'self_balsara_1D'//OBJ2
         write(22,FMT1)TAB,'celij_balsara_ob_1D'//OBJ2
         write(22,FMT1)TAB,'riemann_balsara'//OBJ2
      elseif (ivisc.eq.2) then
         write(22,FMT1)TAB,'celij_visc_1D'//OBJ2
         write(22,FMT1)TAB,'self_visc_1D'//OBJ2
         write(22,FMT1)TAB,'celij_ob_1D'//OBJ2
         write(22,FMT1)TAB,'viscosity_LF_1D'//OBJ2
      elseif (ivisc.eq.1) then
         write(22,FMT1)TAB,'celij_visc_1D'//OBJ2
         write(22,FMT1)TAB,'self_visc_1D'//OBJ2
         write(22,FMT1)TAB,'celij_ob_1D'//OBJ2
         write(22,FMT1)TAB,'viscosity_artificial_1D'//OBJ2
      else
         write(*,*) 'ERROR: ivisc should be 1, 2 or 3'
         write(*,*) 'ivisc=', ivisc
         STOP
      endif
      
      
      write(22,FMT1)TAB,'divide_b_1D'//OBJ2
      write(22,FMT1)TAB,'bottom_1D'//OBJ2
      write(22,FMT1)TAB,'celij_hb_1D'//OBJ2
      write(22,FMT1)TAB,'source_slope_1D'//OBJ2
      write(22,FMT1)TAB,'ac_b_1D'//OBJ2
      write(22,FMT1)TAB,'celij_b_1D'//OBJ2

      if (iMUSCL.eq.1) then
          write(22,FMT1)TAB,'limiter_minmod'//OBJ2
      elseif (iMUSCL.eq.0) then
          write(22,FMT1)TAB,'limiter_noMUSCL'//OBJ2
      else
           write(*,*) 'ERROR: iMUSCL should be 0 or 1'
           write(*,*) 'iMUSCL=', iMUSCL
           STOP
      endif


      write(22,FMT1)TAB,'open_bc_1D'//OBJ2
      write(22,FMT1)TAB,'recount_1D'//OBJ2
      write(22,FMT1)TAB,'celij_dw_ob_1D'//OBJ2
      write(22,FMT1)TAB,'celij_alpha_ob_1D'//OBJ2
      write(22,FMT1)TAB,'divide_ob_1D'//OBJ2
      write(22,FMT1)TAB,'open_bc_pos_1D'//OBJFINAL

      write(22,FMT)      
      
      if(i_compile_opt.eq.4)then
        write(22,FMT)'SPHYSICS_SWE_1D.exe: $(OBJFILES)'

        write(22,FMT)      
        write(22,FMT)'clean:'
        write(22,FMT1)TAB,'del *.obj'
      else
        write(22,FMT)'#'      
        write(22,FMT)'SPHYSICS_SWE_1D: $(objects)'
        write(22,FMT1)TAB,
     &               '$(FC) $(OPTIONS) -o SPHYSICS_SWE_1D $(objects)'
        write(22,FMT)'#'
        write(22,FMT1)TAB,'if [ -d $(bakdir) ]; then \'
        write(22,FMT1)TAB,'echo "execs.bak Directory Exists"; else \'
        write(22,FMT1)TAB,'mkdir $(bakdir); \'
        write(22,FMT1)TAB,'echo "execs.bak Directory Created"; \'
        write(22,FMT1)TAB,'fi'
        write(22,FMT)'#'
        write(22,FMT1)TAB,'if [ -d $(idir) ]; then \'
        write(22,FMT1)TAB,'echo "execs Directory Exists"; else \'
        write(22,FMT1)TAB,'mkdir $(idir); \'
        write(22,FMT1)TAB,'echo "execs Directory Created"; \'
        write(22,FMT1)TAB,'fi'
        write(22,FMT)'#' 
        write(22,FMT1)TAB,'-if [ -f $(idir)/SPHYSICS_SWE_1D ]; then \'
        write(22,FMT1)TAB,'mv -f $(idir)/SPHYSICS_SWE_1D $(bakdir)/; \'
        write(22,FMT1)TAB,'echo Old SPHYSICS_SWE_1D moved to execs.bak'
     &  //' from execs; \'
        write(22,FMT1)TAB,'fi'
        write(22,FMT)'#'
        write(22,FMT1)TAB,'mv SPHYSICS_SWE_1D $(idir)'
        write(22,FMT1)TAB,'echo New SPHYSICS_SWE_1D moved to execs'
        write(22,FMT)'#'
        write(22,FMT)'clean:'
        write(22,FMT1)TAB,'rm *.o'
        write(22,FMT1)TAB,'rm *~' 
        write(22,FMT)'#'
        write(22,FMT)'%.o: %.f'
        write(22,FMT1)TAB,'$(FC) $(OPTIONS) -c -o $@ $<'
      endif
      
      close(22)


      end subroutine
c     ____________________ SUBROUTINE TOCOMPILE_CVF


      subroutine tocompile_cvf(ivisc,idebug,idouble,iMUSCL)
      
      print*,'Invalid Option for compiling'
      stop
      
      end subroutine    

